const ko     = require("knockout");
const _      = require("lodash");
const moment = require("moment");

const {observableAdapter} = require("cattura-ui-library/src/lib/knockoutUtils");

const template = require("./content.pug");

const URI_PROPERTIES = ["imageUri", "player", "staticImageUri", "thumbnailUri", "dashManifestUri"];
const DEFAULT_OPTIONS = {
    onSessionStatusUpdate: _.noop,
    onSessionPause:        _.noop,
    onSessionResume:       _.noop,
    onSessionStop:         _.noop,
    baseUri:               ""
};

function PreviewRecording (options)
{
    if (!(this instanceof PreviewRecording))
    {
        return new PreviewRecording(options);
    }
    
    options = _({})
        .extend(DEFAULT_OPTIONS)
        .extend(options)
        .value();
    
    const apiv2                 = options.apiv2;
    const previewCache          = options.previewCache;
    const onSessionStatusUpdate = (callback => ((message, retval) => {
        if (ko.isWritableObservable(session))
        {
            session(retval.session);
        }
        
        return callback(message, retval);
    }))(options.onSessionStatusUpdate);
    const onSessionPause        = options.onSessionPause;
    const onSessionResume       = options.onSessionResume;
    const onSessionStop         = options.onSessionStop;
    const baseUri               = options.baseUri;
    
    let isPreviewLive  = ko.observable(options.isPreviewLive);
    this.isPreviewLive = isPreviewLive;
    
    let preview = observableAdapter(ko.observable(), {
        readAdapter (value)
        {
            if (!value || !baseUri)
            {
                return value;
            }
            
            return _({}).extend(value).extend({
                streams: _(value.streams).mapValues(stream => {
                    return _(stream).mapValues((value, key) => {
                        return _(URI_PROPERTIES).includes(key) ? baseUri + value
                                                               : value;
                    }).value();
                }).value()
            }).value();
        }
    }, options.preview);
    this.preview = preview;
    
    let selectedSource  = ko.observable();
    this.selectedSource = selectedSource;
    
    let captureRequest  = ko.observable(options.captureRequest);
    this.captureRequest = captureRequest;
    
    let session  = ko.isObservable(options.session) ? options.session
                                                    : ko.observable(options.session);
    this.session = session;
    
    let audioLevels = ko.isObservable(options.audioLevels) ? options.audioLevels :
                                                             ko.observable(options.audioLevels);
    this.audioLevels = audioLevels;
    
    this.audioLevelForSelectedSource = ko.computed(() => {
        const audioLevels = this.audioLevels();
        if (typeof audioLevels !== "object")
        {
            return audioLevels;
        }

        return _(audioLevels).values().head();
    });
    this.elapsedTime = ko.computed(() => {
        const currentSession = session();
        if (!currentSession)
        {
            return -1;
        }
        
        return typeof currentSession.elapsedTime === "function" ? currentSession.elapsedTime()
                                                                : currentSession.elapsedTime;
    });
    
    this.stop = function stop ()
    {
        return apiv2.sessions.stopSession({
            id: captureRequest().mediaPackageID
        }).then(retval => {
            onSessionStatusUpdate(`Recording ${captureRequest().title} stopped successfully.`, retval);
            onSessionStop(retval);
        });
    };

    this.pause = function pause ()
    {
        return apiv2.sessions.pauseSession({
            id: captureRequest().mediaPackageID
        }).then(retval => {
            onSessionStatusUpdate(`Recording ${captureRequest().title} paused successfully.`, retval);
            onSessionPause(retval);
        });
    };

    this.resume = function resume ()
    {
        return apiv2.sessions.resumeSession({
            id: captureRequest().mediaPackageID
        }).then(retval => {
            onSessionStatusUpdate(`Recording ${captureRequest().title} resumed successfully.`, retval);
            onSessionResume(retval);
        });
    };

    this.startLivePreview = function startLivePreview ()
    {
        previewCache.getPreviewForSession(captureRequest()).then(preview => {
            this.preview(preview.live);
            isPreviewLive(true);
        });
    };

    this.stopLivePreview = function stopLivePreview ()
    {
        previewCache.getPreviewForSession(captureRequest()).then(preview => {
            this.preview(preview.thumbnail);
            isPreviewLive(false);
        });
    };
    
    this.startThumbnailPreview = this.stopLivePreview;
    this.render = template;
}

module.exports = PreviewRecording;