const _ = require("lodash");

function PreviewCache (options)
{
    const apiv1 = options.apiv1;
    
    let cachedPreviews = {};

    function makeThumbnailOnlyPreview (preview)
    {
        return _({}).extend(preview).extend({
            streams: _(preview.streams).mapValues(stream => {
                return {
                    title: stream.title,
                    thumbnailUri: stream.thumbnailUri,
                    imageUri: stream.imageUri || stream.staticImageUri
                };
            }).value()
        }).value();
    }

    function getPreviewForSession (session)
    {
        let sessionId = session;
        if (typeof sessionId === "object")
        {
            if (sessionId.captureRequest)
            {
                sessionId = sessionId.captureRequest;
            }

            if (sessionId.mediaPackageID)
            {
                sessionId = sessionId.mediaPackageID;
            }
        }

        if (!sessionId)
        {
            return null;
        }

        if (!(sessionId in cachedPreviews))
        {
            cachedPreviews[sessionId] = cachedPreviews[sessionId] = apiv1.liveStreamViewer.getPreviewForSession({
                id: sessionId
            }).then(preview => {
                return {
                    live: preview,
                    thumbnail: makeThumbnailOnlyPreview(preview)
                };
            }).catch(() => delete cachedPreviews[sessionId]);
        }

        return cachedPreviews[sessionId];
    }
    
    this.getPreviewForSession = getPreviewForSession;
}

module.exports = PreviewCache;